@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2013-2018 Brazil
-  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2018-2019 Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -57,6 +57,9 @@ struct _grn_expr {
     uint32_t codes_curr;
     grn_expr_executor executor;
   } cache;
+
+  grn_obj query_log_tag_prefix;
+  grn_obj *parent;
 };
 
 #define SCAN_ACCESSOR                  (0x01)
