@@ -35,6 +35,52 @@
 #define DECR_DEPTH (DEPTH ? grn_bulk_truncate(ctx, LEVELS, GRN_BULK_VSIZE(LEVELS) - sizeof(uint32_t)) : 0)
 #define INCR_LENGTH (DEPTH ? (GRN_UINT32_VALUE_AT(LEVELS, (DEPTH - 1)) += 2) : 0)
 
+grn_rc
+grn_output_range_normalize(grn_ctx *ctx, int size, int *offset, int *limit)
+{
+  int end;
+  int normalized_offset = *offset;
+  int normalized_limit = *limit;
+
+  if (normalized_limit < 0) {
+    normalized_limit += size + 1;
+    if (normalized_limit < 0) {
+      *offset = 0;
+      *limit = 0;
+      return GRN_TOO_SMALL_LIMIT;
+    }
+  } else if (normalized_limit > size) {
+    normalized_limit = size;
+  }
+
+  if (normalized_offset < 0) {
+    normalized_offset += size;
+    if (normalized_offset < 0) {
+      if (normalized_limit + normalized_offset >= 0) {
+        normalized_limit += normalized_offset;
+        normalized_offset = 0;
+      } else {
+        *offset = 0;
+        *limit = 0;
+        return GRN_TOO_SMALL_OFFSET;
+      }
+    }
+  } else if (normalized_offset != 0 && normalized_offset >= size) {
+    *offset = 0;
+    *limit = 0;
+    return GRN_TOO_LARGE_OFFSET;
+  }
+
+  /* At this point, offset and limit must be zero or positive. */
+  end = normalized_offset + normalized_limit;
+  if (end > size) {
+    normalized_limit -= end - size;
+  }
+  *offset = normalized_offset;
+  *limit = normalized_limit;
+  return GRN_SUCCESS;
+}
+
 static void
 indent(grn_ctx *ctx, grn_obj *outbuf, size_t level)
 {
@@ -723,15 +769,13 @@ grn_text_atoj(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
         break;
       case GRN_ACCESSOR_GET_SCORE :
         {
-          grn_rset_recinfo *ri =
-            (grn_rset_recinfo *)grn_obj_get_value_(ctx, a->obj, id, &vs);
+          double score = grn_table_get_score(ctx, a->obj, id);
           if (grn_ctx_get_command_version(ctx) == GRN_COMMAND_VERSION_1) {
-            int32_t int32_score = ri->score;
+            int32_t int32_score = score;
             GRN_INT32_PUT(ctx, &buf, int32_score);
             buf.header.domain = GRN_DB_INT32;
           } else {
-            double float_score = ri->score;
-            GRN_FLOAT_PUT(ctx, &buf, float_score);
+            GRN_FLOAT_PUT(ctx, &buf, score);
             buf.header.domain = GRN_DB_FLOAT;
           }
         }
@@ -2172,7 +2216,7 @@ transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
                   limit = DEFAULT_LIMIT;
 #undef DEFAULT_LIMIT
                 }
-                grn_normalize_offset_and_limit(ctx, nhits, &offset, &limit);
+                grn_output_range_normalize(ctx, nhits, &offset, &limit);
                 record_n = offset + 1;
                 GRN_TEXT_PUTS(ctx, transformed,
                               "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
